#!/system/bin/sh -x 

if [ -z $1 ]
then
	echo "FATAL: No version supplied, install aborted"
	exit 1
else
	VERSION=$1
fi

INSTALL_READY_FILE=/cloud/install/install.version
INSTALL_DONE_FILE=/cloud/install/install.done
INSTALL_LOCK=/cloud/install/installing
BB=/system/xbin
CACHE=/cache/glass/update

TESTROOTMMC=mmcblk0
cat /proc/cmdline | grep -q $TESTROOTMMC

if [ $? -eq 0 ]
then
	ROOTMMC=$TESTROOTMMC
else
	TESTROOTMMC=mmcblk1
	cat /proc/cmdline | grep -q $TESTROOTMMC

	if [ $? -eq 0 ]
	then
		ROOTMMC=$TESTROOTMMC
	else
		echo "ERROR: Can't find root block device (/dev/mmcblk0 or /dev/mmcblk1), install aborted! (cmdline is '`cat /proc/cmdline`')" 
		RET_CODE=900
		echo $RET_CODE > $INSTALL_DONE_FILE
		exit $RET_CODE
	fi
fi

# find the partition we are running off, and therefore install into the other partition

cd /boot

cmp boot.scr boot-p2.scr

if [ $? -eq 0 ]
then
	# we are running off parition 2
	ROOTFS_PARTITION_DEV=${ROOTMMC}p2
	INSTALL_PARTITION=${ROOTMMC}p3
	INSTALL_PARTITION_LABEL=ROOTFS2
	NEW_BOOT_SCR=boot-p3.scr
else
	cmp boot.scr boot-p3.scr

	if [ $? -eq 0 ]
	then
		# we are running off parition 3
		ROOTFS_PARTITION_DEV=${ROOTMMC}p3
		INSTALL_PARTITION=${ROOTMMC}p2
		INSTALL_PARTITION_LABEL=ROOTFS1
		NEW_BOOT_SCR=boot-p2.scr
	else
		echo "ERROR: Can't find root parition, and therefore mount location of /newrootfs. Not changing boot.scr"
		echo "       Neither cmp boot.scr boot-p2.scr succeeded, not cmp boot.scr boot-p3.scr succeeded"
		RET_CODE=1000
		echo $RET_CODE > $INSTALL_DONE_FILE
		exit $RET_CODE
	fi
fi

# sanity check 1: ROOTFS_PARTITION_DEV MUST be found in kernel command line

cat /proc/cmdline | grep -q $ROOTFS_PARTITION_DEV

if [ $? -ne 0 ]
then
	echo "ERROR: Rootfs partition $ROOTFS_PARTITION_DEV does not exist in kernel command line!  Check that ROOTMMC=$ROOTMMC  is correctly set!"
	echo "       Kernel cmdline is:"
	cat /proc/cmdline

	RET_CODE=1010
	echo $RET_CODE > $INSTALL_DONE_FILE
	exit $RET_CODE
fi
# sanity check 2: INSTALL_PARTITION must NOT be found in kernel command line

cat /proc/cmdline | grep $INSTALL_PARTITION

if [ $? -eq 0 ]
then
	echo "ERROR: Install partition $INSTALL_PARTITION exists in kernel command line!  Not installing to this partition"
	echo "       Kernel cmdline is:"
	cat /proc/cmdline

	RET_CODE=1020
	echo $RET_CODE > $INSTALL_DONE_FILE
	exit $RET_CODE
fi

cd /

echo "Installing rootfs version $VERSION to /dev/block/$INSTALL_PARTITION"

# now download the files

ROOTFS_TAR=$CACHE/android-rootfs-$VERSION.tar.bz2
BOOT_TAR=$CACHE/android-boot-$VERSION.tar.bz2

# check for existence of rootfs tar file

if [ ! -e $ROOTFS_TAR ]
then
	echo "WARNING: File $ROOTFS_TAR not found, rootfs will not be installed"

	INSTALL_ROOTFS=0
	#RET_CODE=1030
	#echo $RET_CODE > $INSTALL_DONE_FILE
	#exit $RET_CODE
else
	INSTALL_ROOTFS=1
fi

# check for existence of boot tar file

if [ ! -e $BOOT_TAR ]
then
	echo "WARNING: Update failed! File $BOOT_TAR not found!"

	INSTALL_BOOT=0

	if [ $INSTALL_ROOTS -eq 0 ]
	then
		echo "Install failed because neither rootfs tar nor boot tar were found!"

		RET_CODE=1040
		echo $RET_CODE > $INSTALL_DONE_FILE
		exit $RET_CODE
	fi
else
	INSTALL_BOOT=1
fi

# now actually do the install

if [ $INSTALL_ROOTFS -eq 1 ]
then
	# install rootfs

	DEV=/dev/block/${INSTALL_PARTITION}

	killall httpd

	$BB/umount -f /newrootfs

	if [ $? -ne 0 ]
	then
		echo "FATAL: Can't unmount /newrootfs, so that partition $DEV might be re-formatted"
		RET_CODE=1050
		echo $RET_CODE > $INSTALL_DONE_FILE
		exit $RET_CODE
	fi

	mke2fs  -j -L $INSTALL_PARTITION_LABEL $DEV

	if [ $? -ne 0 ]
	then
		echo "FATAL: Can't format ext3 partition on $DEV"
		RET_CODE=1060
		echo $RET_CODE > $INSTALL_DONE_FILE
		exit $RET_CODE
	fi

	mount -t ext3 $DEV /newrootfs

	if [ $? -ne 0 ]
	then
		echo "FATAL: Can't mount ext3 partition $DEV on to /newrootfs"
		RET_CODE=1070
		echo $RET_CODE > $INSTALL_DONE_FILE
		exit $RET_CODE
	fi

	cd /newrootfs

	if [ $? -ne 0 ]
	then
		echo "FATAL: Can't cd to /newrootfs, install aborted"
		RET_CODE=1080
		echo $RET_CODE > $INSTALL_DONE_FILE
		exit $RET_CODE
	fi

	echo "Extracting $ROOTFS_TAR in /newrootfs..."

	$BB/tar jxf $ROOTFS_TAR

	if [ $? -ne 0 ]
	then
		echo "FATAL: Can't run tar jxf $ROOTFS_TAR in /newrootfs, install aborted"
		RET_CODE=1090
		echo $RET_CODE > $INSTALL_DONE_FILE
		exit $RET_CODE
	fi
	#Reset flag after Software update	
	echo 1 > cloud/lastrebootinfo

	#Copying handsetcordtype to newrootfs after Software update
	if [ -e /cloud/handsetcordtype ]
	then
    	cp /cloud/handsetcordtype cloud/
	fi

	$BB/sync
else
	echo "Not installing a new rootfs because rootfs tar was not downloaded!"
fi

# now handle boot files

if [ $INSTALL_BOOT -eq 1 ]
then
	cd $CACHE

		if [ $? -ne 0 ]
		then
			echo "FATAL: Can't cd to $CACHE, install aborted"
			RET_CODE=1110
			echo $RET_CODE > $INSTALL_DONE_FILE
			exit $RET_CODE
		fi

	$BB/rm -rf $CACHE/boot

		if [ $? -ne 0 ]
		then
			echo "FATAL: Can't rm -rf $CACHE/boot, install aborted"
			RET_CODE=1120
			echo $RET_CODE > $INSTALL_DONE_FILE
			exit $RET_CODE
		fi

	mkdir $CACHE/boot

		if [ $? -ne 0 ]
		then
			echo "FATAL: Can't mkdir to $CACHE/boot, install aborted"
			RET_CODE=1130
			echo $RET_CODE > $INSTALL_DONE_FILE
			exit $RET_CODE
		fi

	cd $CACHE/boot

		if [ $? -ne 0 ]
		then
			echo "FATAL: Can't cd to $CACHE/boot, install aborted"
			RET_CODE=1140
			echo $RET_CODE > $INSTALL_DONE_FILE
			exit $RET_CODE
		fi

	$BB/tar jxf $BOOT_TAR

		if [ $? -ne 0 ]
		then
			echo "FATAL: Can't run tar jxf $ROOTFS_TAR in $CACHE/boot, install aborted"
			RET_CODE=1150
			echo $RET_CODE > $INSTALL_DONE_FILE
			exit $RET_CODE
		fi

	#Below lines not needed as we may update the boot scripts.
	#$BB/cp -p /boot/boot-p2.scr $CACHE/boot
	#$BB/cp -p /boot/boot-p3.scr $CACHE/boot

	$BB/umount -f /boot

		if [ $? -ne 0 ]
		then
			echo "FATAL: Can't umount /boot, install aborted"
			RET_CODE=1160
			echo $RET_CODE > $INSTALL_DONE_FILE
			exit $RET_CODE
		fi

	sleep 1

	BOOT_DEV=/dev/block/${ROOTMMC}p1

	echo "Formatting FAT32 partition $BOOT_DEV ..."

	mkdosfs -F 32 -n BOOT $BOOT_DEV

		if [ $? -ne 0 ]
		then
			echo "FATAL: Can't format $BOOT_DEV, install aborted, phone WILL NOT BOOT!!!!"
			RET_CODE=1170
			exit $RET_CODE
		fi

	mount -t vfat /dev/block/${ROOTMMC}p1 /boot

		if [ $? -ne 0 ]
		then
			MSG="FATAL: Can't mount $BOOT_DEV on /boot, install aborted, phone WILL NOT BOOT!!!!"
			RET_CODE=1180
			exit $RET_CODE
		fi

	cd /boot

		if [ $? -ne 0 ]
		then
			echo "FATAL: Can't cd to /boot after umount, mkdosfs, and remount, install aborted, phone WILL NOT BOOT!!!!"
			RET_CODE=1190
			echo $RET_CODE > $INSTALL_DONE_FILE
			exit $RET_CODE
		fi
	
	echo "Copying boot files to /boot"

	$BB/cp -p $CACHE/boot/MLO /boot
	$BB/cp -p $CACHE/boot/u-boot.bin /boot
	$BB/cp -p $CACHE/boot/uImage /boot
	$BB/cp    $CACHE/boot/$NEW_BOOT_SCR /boot/boot.scr
	$BB/cp -p $CACHE/boot/boot-p2.scr /boot
	$BB/cp -p $CACHE/boot/boot-p3.scr /boot

	sleep 1

	$BB/sync
else
	echo "Not installing a new boot because boot tar was not downloaded!"
fi

if [ $INSTALL_ROOTFS -eq 1 ]
then
	# switch the boot files so we boot off $INSTALL_PARTITION

	echo "Copying /boot/$NEW_BOOT_SCR to /boot/boot.scr"

	cp /boot/${NEW_BOOT_SCR}  /boot/boot.scr
else
	echo "Not switching rootfs partition because rootfs tar was not downloaded"
fi

# success
echo "Sucessful install: ROOTFS install = $INSTALL_ROOTFS, BOOT install = $INSTALL_BOOT"

RET_CODE=0
echo $RET_CODE > $INSTALL_DONE_FILE
exit $RET_CODE
